@@ -1,9 +1,8 @@
 # -*- coding: utf-8 -*-
 '''
-We wanna be free to ride our machines without being hassled by The Man!
-And we wanna get loaded!
-And we wanna have a good time!
-And that's what we are gonna do. We are gonna have a good time...
+The Salt loader is the core to Salt's plugin system, the loader scans
+directories for python loadable code and organizes the code into the
+plugin interfaces used by Salt.
 '''
 
 # Import python libs
@@ -122,12 +121,14 @@ def static_loader(
 def _module_dirs(
         opts,
         ext_type,
-        tag,
+        tag=None,
         int_type=None,
         ext_dirs=True,
         ext_type_dirs=None,
         base_path=None,
         ):
+    if tag is None:
+        tag = ext_type
     sys_types = os.path.join(base_path or SALT_BASE_PATH, int_type or ext_type)
     ext_types = os.path.join(opts['extension_modules'], ext_type)
 
@@ -164,7 +165,6 @@ def minion_mods(
         context=None,
         utils=None,
         whitelist=None,
-        include_errors=False,
         initial_load=False,
         loaded_base_name=None,
         notify=False,
@@ -187,7 +187,6 @@ def minion_mods(
                             configuration.
 
     :param list whitelist: A list of modules which should be whitelisted.
-    :param bool include_errors: Deprecated flag! Unused.
     :param bool initial_load: Deprecated flag! Unused.
     :param str loaded_base_name: A string marker for the loaded base name.
     :param bool notify: Flag indicating that an event should be fired upon
@@ -281,7 +280,7 @@ def engines(opts, functions, runners, proxy=None):
             '__runners__': runners,
             '__proxy__': proxy}
     return LazyLoader(
-        _module_dirs(opts, 'engines', 'engines'),
+        _module_dirs(opts, 'engines'),
         opts,
         tag='engines',
         pack=pack,
@@ -293,7 +292,7 @@ def proxy(opts, functions=None, returners=None, whitelist=None):
     Returns the proxy module for this salt-proxy-minion
     '''
     ret = LazyLoader(
-        _module_dirs(opts, 'proxy', 'proxy'),
+        _module_dirs(opts, 'proxy'),
         opts,
         tag='proxy',
         pack={'__salt__': functions, '__ret__': returners},
@@ -322,7 +321,7 @@ def utils(opts, whitelist=None, context=None):
     Returns the utility modules
     '''
     return LazyLoader(
-        _module_dirs(opts, 'utils', 'utils', ext_type_dirs='utils_dirs'),
+        _module_dirs(opts, 'utils', ext_type_dirs='utils_dirs'),
         opts,
         tag='utils',
         whitelist=whitelist,
@@ -334,9 +333,7 @@ def pillars(opts, functions, context=None):
     '''
     Returns the pillars modules
     '''
-    if context is None:
-        context = {}
-    ret = LazyLoader(_module_dirs(opts, 'pillar', 'pillar'),
+    ret = LazyLoader(_module_dirs(opts, 'pillar'),
                      opts,
                      tag='pillar',
                      pack={'__salt__': functions,
@@ -367,7 +364,7 @@ def wheels(opts, whitelist=None):
     Returns the wheels modules
     '''
     return LazyLoader(
-        _module_dirs(opts, 'wheel', 'wheel'),
+        _module_dirs(opts, 'wheel'),
         opts,
         tag='wheel',
         whitelist=whitelist,
@@ -382,7 +379,7 @@ def outputters(opts):
     :returns: LazyLoader instance, with only outputters present in the keyspace
     '''
     ret = LazyLoader(
-        _module_dirs(opts, 'output', 'output', ext_type_dirs='outputter_dirs'),
+        _module_dirs(opts, 'output', ext_type_dirs='outputter_dirs'),
         opts,
         tag='output',
     )
@@ -399,7 +396,7 @@ def serializers(opts):
     :returns: LazyLoader instance, with only serializers present in the keyspace
     '''
     return LazyLoader(
-        _module_dirs(opts, 'serializers', 'serializers'),
+        _module_dirs(opts, 'serializers'),
         opts,
         tag='serializers',
     )
@@ -413,7 +410,7 @@ def auth(opts, whitelist=None):
     :returns: LazyLoader
     '''
     return LazyLoader(
-        _module_dirs(opts, 'auth', 'auth'),
+        _module_dirs(opts, 'auth'),
         opts,
         tag='auth',
         whitelist=whitelist,
@@ -425,7 +422,7 @@ def fileserver(opts, backends):
     '''
     Returns the file server modules
     '''
-    return LazyLoader(_module_dirs(opts, 'fileserver', 'fileserver'),
+    return LazyLoader(_module_dirs(opts, 'fileserver'),
                       opts,
                       tag='fileserver',
                       whitelist=backends,
@@ -437,7 +434,7 @@ def roster(opts, whitelist=None):
     Returns the roster modules
     '''
     return LazyLoader(
-        _module_dirs(opts, 'roster', 'roster'),
+        _module_dirs(opts, 'roster'),
         opts,
         tag='roster',
         whitelist=whitelist,
@@ -449,7 +446,7 @@ def thorium(opts, functions, runners):
     Load the thorium runtime modules
     '''
     pack = {'__salt__': functions, '__runner__': runners, '__context__': {}}
-    ret = LazyLoader(_module_dirs(opts, 'thorium', 'thorium'),
+    ret = LazyLoader(_module_dirs(opts, 'thorium'),
             opts,
             tag='thorium',
             pack=pack)
@@ -474,7 +471,7 @@ def states(opts, functions, utils, serializers, whitelist=None):
         statemods = salt.loader.states(__opts__, None, None)
     '''
     ret = LazyLoader(
-        _module_dirs(opts, 'states', 'states'),
+        _module_dirs(opts, 'states'),
         opts,
         tag='states',
         pack={'__salt__': functions},
@@ -495,10 +492,11 @@ def beacons(opts, functions, context=None):
                             keys and funcs as values.
     '''
     return LazyLoader(
-        _module_dirs(opts, 'beacons', 'beacons'),
+        _module_dirs(opts, 'beacons'),
         opts,
         tag='beacons',
         pack={'__context__': context, '__salt__': functions},
+        virtual_funcs=['__validate__'],
     )
 
 
@@ -531,7 +529,6 @@ def log_handlers(opts):
         _module_dirs(
             opts,
             'log_handlers',
-            'log_handlers',
             int_type='handlers',
             base_path=os.path.join(SALT_BASE_PATH, 'log'),
         ),
@@ -549,7 +546,6 @@ def ssh_wrapper(opts, functions=None, context=None):
         _module_dirs(
             opts,
             'wrapper',
-            'wrapper',
             base_path=os.path.join(SALT_BASE_PATH, os.path.join('client', 'ssh')),
         ),
         opts,
@@ -584,7 +580,7 @@ def render(opts, functions, states=None):
     )
     rend = FilterDictWrapper(ret, '.render')
 
-    if not check_render_pipe_str(opts['renderer'], rend):
+    if not check_render_pipe_str(opts['renderer'], rend, opts['renderer_blacklist'], opts['renderer_whitelist']):
         err = ('The renderer {0} is unavailable, this error is often because '
                'the needed software is unavailable'.format(opts['renderer']))
         log.critical(err)
@@ -635,7 +631,7 @@ def grains(opts, force_refresh=False, proxy=None):
         __grains__ = salt.loader.grains(__opts__)
         print __grains__['id']
     '''
-    # if we hae no grains, lets try loading from disk (TODO: move to decorator?)
+    # if we have no grains, lets try loading from disk (TODO: move to decorator?)
     cfn = os.path.join(
         opts['cachedir'],
         'grains.cache.p'
@@ -806,14 +802,17 @@ def call(fun, **kwargs):
     return funcs[fun](*args)
 
 
-def runner(opts):
+def runner(opts, utils=None):
     '''
     Directly call a function inside a loader directory
     '''
+    if utils is None:
+        utils = {}
     ret = LazyLoader(
         _module_dirs(opts, 'runners', 'runner', ext_type_dirs='runner_dirs'),
         opts,
         tag='runners',
+        pack={'__utils__': utils},
     )
     # TODO: change from __salt__ to something else, we overload __salt__ too much
     ret.pack['__salt__'] = ret
@@ -836,7 +835,7 @@ def sdb(opts, functions=None, whitelist=None):
     Make a very small database call
     '''
     return LazyLoader(
-        _module_dirs(opts, 'sdb', 'sdb'),
+        _module_dirs(opts, 'sdb'),
         opts,
         tag='sdb',
         pack={'__sdb__': functions},
@@ -854,7 +853,6 @@ def pkgdb(opts):
         _module_dirs(
             opts,
             'pkgdb',
-            'pkgdb',
             base_path=os.path.join(SALT_BASE_PATH, 'spm')
         ),
         opts,
@@ -872,7 +870,6 @@ def pkgfiles(opts):
         _module_dirs(
             opts,
             'pkgfiles',
-            'pkgfiles',
             base_path=os.path.join(SALT_BASE_PATH, 'spm')
         ),
         opts,
@@ -914,7 +911,7 @@ def netapi(opts):
     Return the network api functions
     '''
     return LazyLoader(
-        _module_dirs(opts, 'netapi', 'netapi'),
+        _module_dirs(opts, 'netapi'),
         opts,
         tag='netapi',
     )
@@ -932,6 +929,18 @@ def executors(opts, functions=None, context=None):
     )
 
 
+def cache(opts, serial):
+    '''
+    Returns the returner modules
+    '''
+    return LazyLoader(
+        _module_dirs(opts, 'cache', 'cache'),
+        opts,
+        tag='cache',
+        pack={'__opts__': opts, '__context__': {'serial': serial}},
+    )
+
+
 def _generate_module(name):
     if name in sys.modules:
         return
@@ -979,10 +988,26 @@ class FilterDictWrapper(MutableMapping):
 
 class LazyLoader(salt.utils.lazy.LazyDict):
     '''
-    Goals here:
-        - lazy loading
-        - minimize disk usage
+    A pseduo-dictionary which has a set of keys which are the
+    name of the module and function, delimited by a dot. When
+    the value of the key is accessed, the function is then loaded
+    from disk and into memory.
 
+    .. note::
+
+        Iterating over keys will cause all modules to be loaded.
+
+    :param list module_dirs: A list of directories on disk to search for modules
+    :param opts dict: The salt options dictionary.
+    :param tag str': The tag for the type of module to load
+    :param func mod_type_check: A function which can be used to verify files
+    :param dict pack: A dictionary of function to be packed into modules as they are loaded
+    :param list whitelist: A list of modules to whitelist
+    :param bool virtual_enable: Whether or not to respect the __virtual__ function when loading modules.
+    :param str virtual_funcs: The name of additional functions in the module to call to verify its functionality.
+                                If not true, the module will not load.
+    :returns: A LazyLoader object which functions as a dictionary. Keys are 'module.function' and values
+    are function references themselves which are loaded on-demand.
     # TODO:
         - move modules_max_memory into here
         - singletons (per tag)
@@ -1000,7 +1025,8 @@ class LazyLoader(salt.utils.lazy.LazyDict):
                  whitelist=None,
                  virtual_enable=True,
                  static_modules=None,
-                 proxy=None
+                 proxy=None,
+                 virtual_funcs=None,
                  ):  # pylint: disable=W0231
         '''
         In pack, if any of the values are None they will be replaced with an
@@ -1037,6 +1063,10 @@ class LazyLoader(salt.utils.lazy.LazyDict):
         self.loaded_files = set()  # TODO: just remove them from file_mapping?
         self.static_modules = static_modules if static_modules else []
 
+        if virtual_funcs is None:
+            virtual_funcs = []
+        self.virtual_funcs = virtual_funcs
+
         self.disabled = set(self.opts.get('disable_{0}s'.format(self.tag), []))
 
         self.refresh_file_mapping()
@@ -1132,7 +1162,8 @@ class LazyLoader(salt.utils.lazy.LazyDict):
         self.suffix_map[''] = ('', '', imp.PKG_DIRECTORY)
 
         # create mapping of filename (without suffix) to (path, suffix)
-        self.file_mapping = {}
+        # The files are added in order of priority, so order *must* be retained.
+        self.file_mapping = salt.utils.odict.OrderedDict()
 
         for mod_dir in self.module_dirs:
             files = []
@@ -1175,9 +1206,11 @@ class LazyLoader(salt.utils.lazy.LazyDict):
                         curr_ext = self.file_mapping[f_noext][1]
                         #log.debug("****** curr_ext={0} ext={1} suffix_order={2}".format(curr_ext, ext, suffix_order))
                         if '' in (curr_ext, ext) and curr_ext != ext:
-                            log.error('Module/package collision: {0!r} and {1!r}'.format(
-                                fpath, self.file_mapping[f_noext][0]
-                            ))
+                            log.error(
+                                'Module/package collision: \'%s\' and \'%s\'',
+                                fpath,
+                                self.file_mapping[f_noext][0]
+                            )
                         if not curr_ext or suffix_order.index(ext) >= suffix_order.index(curr_ext):
                             continue  # Next filename
 
@@ -1210,11 +1243,11 @@ class LazyLoader(salt.utils.lazy.LazyDict):
         '''
         if '__grains__' not in self.pack:
             self.context_dict['grains'] = opts.get('grains', {})
-            self.pack['__grains__'] = salt.utils.context.NamespacedDictWrapper(self.context_dict, 'grains')
+            self.pack['__grains__'] = salt.utils.context.NamespacedDictWrapper(self.context_dict, 'grains', override_name='grains')
 
         if '__pillar__' not in self.pack:
             self.context_dict['pillar'] = opts.get('pillar', {})
-            self.pack['__pillar__'] = salt.utils.context.NamespacedDictWrapper(self.context_dict, 'pillar')
+            self.pack['__pillar__'] = salt.utils.context.NamespacedDictWrapper(self.context_dict, 'pillar', override_name='pillar')
 
         mod_opts = {}
         for key, val in list(opts.items()):
@@ -1300,7 +1333,10 @@ class LazyLoader(salt.utils.lazy.LazyDict):
 
         except IOError:
             raise
-        except ImportError:
+        except ImportError as exc:
+            if 'magic number' in str(exc):
+                log.warning('Failed to import {0} {1}. Bad magic number. If migrating '
+                        'from Python2 to Python3, remove all .pyc files and try again.'.format(self.tag, name))
             log.debug(
                 'Failed to import {0} {1}:\n'.format(
                     self.tag, name
@@ -1361,23 +1397,25 @@ class LazyLoader(salt.utils.lazy.LazyDict):
         # if virtual modules are enabled, we need to look for the
         # __virtual__() function inside that module and run it.
         if self.virtual_enable:
-            (virtual_ret, module_name, virtual_err) = self.process_virtual(
-                mod,
-                module_name,
-            )
-            if virtual_err is not None:
-                log.trace('Error loading {0}.{1}: {2}'.format(self.tag,
-                                                              module_name,
-                                                              virtual_err,
-                                                              ))
+            virtual_funcs_to_process = ['__virtual__'] + self.virtual_funcs
+            for virtual_func in virtual_funcs_to_process:
+                (virtual_ret, module_name, virtual_err) = self.process_virtual(
+                    mod,
+                    module_name,
+                )
+                if virtual_err is not None:
+                    log.trace('Error loading {0}.{1}: {2}'.format(self.tag,
+                                                                  module_name,
+                                                                  virtual_err,
+                                                                  ))
 
-            # if process_virtual returned a non-True value then we are
-            # supposed to not process this module
-            if virtual_ret is not True:
-                # If a module has information about why it could not be loaded, record it
-                self.missing_modules[module_name] = virtual_err
-                self.missing_modules[name] = virtual_err
-                return False
+                # if process_virtual returned a non-True value then we are
+                # supposed to not process this module
+                if virtual_ret is not True and module_name not in self.missing_modules:
+                    # If a module has information about why it could not be loaded, record it
+                    self.missing_modules[module_name] = virtual_err
+                    self.missing_modules[name] = virtual_err
+                    return False
 
         # If this is a proxy minion then MOST modules cannot work. Therefore, require that
         # any module that does work with salt-proxy-minion define __proxyenabled__ as a list
@@ -1401,7 +1439,14 @@ class LazyLoader(salt.utils.lazy.LazyDict):
                     module_name
                 )
             )
-        mod_dict = self.mod_dict_class()
+
+        # If we had another module by the same virtual name, we should put any
+        # new functions under the existing dictionary.
+        if module_name in self.loaded_modules:
+            mod_dict = self.loaded_modules[module_name]
+        else:
+            mod_dict = self.mod_dict_class()
+
         for attr in getattr(mod, '__load__', dir(mod)):
             if attr.startswith('_'):
                 # private functions are skipped
@@ -1418,11 +1463,15 @@ class LazyLoader(salt.utils.lazy.LazyDict):
             # It default's of course to the found callable attribute name
             # if no alias is defined.
             funcname = getattr(mod, '__func_alias__', {}).get(attr, attr)
+            full_funcname = '{0}.{1}'.format(module_name, funcname)
             # Save many references for lookups
-            self._dict['{0}.{1}'.format(module_name, funcname)] = func
-            setattr(mod_dict, funcname, func)
-            mod_dict[funcname] = func
-            self._apply_outputter(func, mod)
+            # Careful not to overwrite existing (higher priority) functions
+            if full_funcname not in self._dict:
+                self._dict[full_funcname] = func
+            if funcname not in mod_dict:
+                setattr(mod_dict, funcname, func)
+                mod_dict[funcname] = func
+                self._apply_outputter(func, mod)
 
         # enforce depends
         try:
@@ -1498,7 +1547,7 @@ class LazyLoader(salt.utils.lazy.LazyDict):
             if func.__name__ in outp:
                 func.__outputter__ = outp[func.__name__]
 
-    def process_virtual(self, mod, module_name):
+    def process_virtual(self, mod, module_name, virtual_func='__virtual__'):
         '''
         Given a loaded module and its default name determine its virtual name
 
@@ -1529,7 +1578,7 @@ class LazyLoader(salt.utils.lazy.LazyDict):
             if hasattr(mod, '__virtual__') and inspect.isfunction(mod.__virtual__):
                 try:
                     start = time.time()
-                    virtual = mod.__virtual__()
+                    virtual = getattr(mod, virtual_func)()
                     if isinstance(virtual, tuple):
                         error_reason = virtual[1]
                         virtual = virtual[0]
